/*************** <lalVerbatim file="CorrelationCV"> ****************
Author: Yakushin, Igor
$Id: Correlation.c,v 1.1 2002/09/11 20:29:17 jolien Exp $
**************** </lalVerbatim> *************************************/
/**** <lalLaTeX>
 * \subsection{Module \texttt{Correlation.c}}
 * \vfill{\footnotesize\input{CorrelationCV}}
 **** </lalLaTeX> */
#include <stdio.h>
#include <lal/LALStdlib.h>
#include <lal/LALCorrelation.h>
#include <lal/Units.h>
#include <Correlation.h>
#include <BuildDB.h>

NRCSID( CORRELATIONC, "$Id" );

void LALInitSearch(LALStatus *status,
		   void **searchParams,
		   LALInitSearchParams *initSearchParams
		   )
{
  INT4 ac;
  CHAR **av;
  INT4 i;
  int tmp;
  CorrelationParamsLW *params;

  INITSTATUS( status, "LALInitSearch", CORRELATIONC );

  /*Memory allocation*/

  params=(CorrelationParamsLW *)LALMalloc(sizeof(CorrelationParamsLW));

  params->params=(CorrelationParams *)LALMalloc(sizeof(CorrelationParams));
  params->input=(InputCorrelation *)LALMalloc(sizeof(InputCorrelation));

  params->input->one=(REAL4TimeSeries*)LALMalloc(sizeof(REAL4TimeSeries));
  params->input->two=(REAL4TimeSeries*)LALMalloc(sizeof(REAL4TimeSeries));

  params->input->one->data=(REAL4Sequence*)LALMalloc(sizeof(REAL4Sequence));
  params->input->two->data=(REAL4Sequence*)LALMalloc(sizeof(REAL4Sequence));

  *searchParams=params;

  /*Defaults*/

  params->params->maxTimeShiftNan=1000;
  strcpy(params->nameOne,"one");
  strcpy(params->nameTwo,"two");
  params->threshold=0.6;

  params->ncalls=0;

  /*Default parameters are overwritten if specified in -filterparams */

  ac=initSearchParams->argc;
  av=initSearchParams->argv;

  for(i=1;i<ac;i++){
    if(strcmp(av[i],"-maxTimeShiftNan")==0){
      params->params->maxTimeShiftNan=atof(av[i+1]);
#ifdef DEBUG
      printf("Init: %s %s\n",av[i],av[i+1]);
      printf("maxTimeShiftNan=%f\n",params->params->maxTimeShiftNan);
#endif /*DEBUG*/
    }
    if(strcmp(av[i],"-ch1")==0){
      strcpy(params->nameOne,av[i+1]);
#ifdef DEBUG
      printf("Init: %s %s\n",av[i],av[i+1]);
#endif /*DEBUG*/
    }
    if(strcmp(av[i],"-ch2")==0){
      strcpy(params->nameTwo,av[i+1]);
#ifdef DEBUG
      printf("Init: %s %s\n",av[i],av[i+1]);
#endif /*DEBUG*/      
    }
    if(strcmp(av[i],"-threshold")==0){
      params->threshold=(REAL4)atof(av[i+1]);
#ifdef DEBUG
      printf("Init: %s %s\n",av[i],av[i+1]);
#endif /*DEBUG*/      
    }
  }

#ifdef DEBUG
  printf("Init: startTime=%ld\n",initSearchParams->startTime);
  printf("Init: dataDuration=%ld\n",initSearchParams->dataDuration);
  printf("Init: realtimeRatio=%f\n",initSearchParams->realtimeRatio);
  printf("Init: nodeClass=%d\n",initSearchParams->nodeClass);
  printf("--------------------\n\n");
  fflush(stdout);
#endif /*DEBUG*/
  
  RETURN( status );
}

void LALConditionData(LALStatus *status,
		      LALSearchInput *inout,
		      void *searchParams,
		      LALMPIParams *mpiParams
		      )
{
  CorrelationParamsLW *params;
  INT4 i;

  INITSTATUS( status, "LALConditionData", CORRELATIONC );
  ATTATCHSTATUSPTR( status );

  mpiParams=NULL;

  params=(CorrelationParamsLW*)searchParams;

  /* Checking data consistency */

  if(inout->numberSequences!=2){
    ABORT(status,CORRELATIONH_ENSEQ,CORRELATIONH_MSGENSEQ);
  }

  if(strcmp(inout->sequences[0].units,inout->sequences[1].units)!=0){
    ABORT(status,CORRELATIONH_EUNITS,CORRELATIONH_MSGEUNITS);
  }

  if(inout->sequences[0].type!=inout->sequences[1].type ||
     inout->sequences[0].type!=real_4){
    ABORT(status,CORRELATIONH_ETYPES,CORRELATIONH_MSGETYPES);
  }
  
  if(inout->sequences[0].space!=timeD || inout->sequences[1].space!=timeD){
    ABORT(status,CORRELATIONH_ESPACES,CORRELATIONH_MSGESPACES);
  }

  if(inout->sequences[0].range.dTime.numberSamples!=inout->sequences[0].range.dTime.numberSamples ||
     inout->sequences[0].numberDimensions!=inout->sequences[1].numberDimensions || 
     inout->sequences[0].dimensions[0]!=inout->sequences[1].dimensions[0]){
    ABORT(status,CORRELATIONH_EDIMS,CORRELATIONH_MSGEDIMS);
  }

  if(inout->sequences[0].range.dTime.startSec!=inout->sequences[1].range.dTime.startSec){
    ABORT(status,CORRELATIONH_ESTARTS,CORRELATIONH_MSGESTARTS);
  }
  if(inout->sequences[0].range.dTime.startNan!=inout->sequences[1].range.dTime.startNan){
    ABORT(status,CORRELATIONH_ESTARTN,CORRELATIONH_MSGESTARTN);
  }

  if(inout->sequences[0].range.dTime.stopSec!=inout->sequences[1].range.dTime.stopSec){
    ABORT(status,CORRELATIONH_ESTOPS,CORRELATIONH_MSGESTOPS);
  }
  if(inout->sequences[0].range.dTime.stopNan!=inout->sequences[1].range.dTime.stopNan){
    ABORT(status,CORRELATIONH_ESTOPN,CORRELATIONH_MSGESTOPN);
  }

  if(inout->sequences[0].range.dTime.timeStepSize!=inout->sequences[1].range.dTime.timeStepSize){
    ABORT(status,CORRELATIONH_ETIMESTEP,CORRELATIONH_MSGETIMESTEP);
  }

  params->input->one->epoch.gpsSeconds=inout->sequences[0].range.dTime.startSec;
  params->input->two->epoch.gpsSeconds=inout->sequences[1].range.dTime.startSec;
  params->input->one->epoch.gpsNanoSeconds=inout->sequences[0].range.dTime.startNan;
  params->input->two->epoch.gpsNanoSeconds=inout->sequences[1].range.dTime.startNan;
  params->input->one->deltaT=inout->sequences[0].range.dTime.timeStepSize;
  params->input->two->deltaT=inout->sequences[1].range.dTime.timeStepSize;
  params->input->one->data->length=inout->sequences[0].range.dTime.numberSamples;
  params->input->two->data->length=inout->sequences[1].range.dTime.numberSamples;
  params->input->one->data->data=inout->sequences[0].data.real4;
  params->input->two->data->data=inout->sequences[1].data.real4;
  params->input->one->f0=inout->sequences[0].range.dTime.baseFreq;
  params->input->two->f0=inout->sequences[1].range.dTime.baseFreq;

#ifdef DEBUG
  printf("Condition: numberSequences=%d\n\n",inout->numberSequences);
  printf("---\n\n");
  for(i=0;i<inout->numberSequences;i++){
    printf("Condition: sequences[%d].name=%s\n",i,inout->sequences[i].name);
    printf("Condition: sequences[%d].units=%s\n",i,inout->sequences[i].units);
    printf("Condition: sequences[%d].comment=%s\n",i,inout->sequences[i].comment);
    printf("Condition: sequences[%d].numberDimensions=%d\n",i,inout->sequences[i].numberDimensions);
    printf("Condition: sequences[%d].dimensions[0]=%d\n",i,inout->sequences[i].dimensions[0]);
    printf("Condition: sequences[%d].type=%d\n",i,inout->sequences[i].type);

    printf("Condition: one->deltaT=%f\n",params->input->one->deltaT);
    printf("Condition: two->deltaT=%f\n",params->input->two->deltaT);

    switch(inout->sequences[i].type){
    case boolean_lu:
      printf("boolean_lu\n");
      break;
    case char_s:
      printf("char_s\n");
      break;
    case char_u:
      printf("char_u\n");
      break;
    case int_2s:
      printf("int_2s\n");
      break;
    case int_2u:
      printf("int_2u\n");
      break;
    case int_4s:
      printf("int_4s\n");
      break;
    case int_4u:
      printf("int_4u\n");
      break;
    case int_8s:
      printf("int_8s\n");
      break;
    case int_8u:
      printf("int_8u\n");
      break;
    case real_4:
      printf("real_4\n");
      break;
    case real_8:
      printf("real_8\n");
      break;
    case complex_8:
      printf("complex_8\n");
      break;
    case complex_16:
      printf("complex_16\n");
      break;
    case char_s_ptr:
      printf("char_s_ptr\n");
      break;
    case char_u_ptr:
      printf("char_u_ptr\n");
      break;
    default:
      printf("Unrecognized datatype\n");
    }

    printf("Condition: sequences[%d].space=%d\n",i,inout->sequences[i].space);

    switch(inout->sequences[i].space){
    case timeD: 
      printf("time\n");
      printf("numberSamples=%ld\n",inout->sequences[i].range.dTime.numberSamples);
      printf("startSec=%d\n",inout->sequences[i].range.dTime.startSec);
      printf("startNan=%d\n",inout->sequences[i].range.dTime.startNan);
      printf("stopSec=%d\n",inout->sequences[i].range.dTime.stopSec);
      printf("stopNan=%d\n",inout->sequences[i].range.dTime.stopNan);
      printf("timeStepSize=%g\n",inout->sequences[i].range.dTime.timeStepSize);
      printf("baseFreq=%g\n",inout->sequences[i].range.dTime.baseFreq);
      printf("phase=%f\n",inout->sequences[i].range.dTime.phase);
      printf("numberChannels=%d\n",inout->sequences[i].range.dTime.numberChannels);
      printf("numberDetectors=%d\n",inout->sequences[i].range.dTime.numberDetectors);
      break;
    case freqD: 
      printf("freq\n");
      printf("numberSamples=%ld\n",inout->sequences[i].range.dFreq.numberSamples);
      printf("startSec=%d\n",inout->sequences[i].range.dFreq.gpsStartTimeSec);
      printf("startNan=%d\n",inout->sequences[i].range.dFreq.gpsStartTimeNan);
      printf("stopSec=%d\n",inout->sequences[i].range.dFreq.gpsStopTimeSec);
      printf("stopNan=%d\n",inout->sequences[i].range.dFreq.gpsStopTimeNan);
      printf("startFreq=%g\n",inout->sequences[i].range.dFreq.startFreq);
      printf("stopFreq=%g\n",inout->sequences[i].range.dFreq.stopFreq);
      printf("freqStepSize=%g\n",inout->sequences[i].range.dFreq.freqStepSize);
      printf("baseFreq=%g\n",inout->sequences[i].range.dFreq.baseFreq);      
      printf("numberChannels=%d\n",inout->sequences[i].range.dTime.numberChannels);
      printf("numberDetectors=%d\n",inout->sequences[i].range.dTime.numberDetectors);
      break;
    case bothD: 
      printf("both\n");
      break;
    case databaseD:
      printf("database\n");
      break;
    default:
      printf("default\n");
    }
    printf("--------------------\n\n");
    fflush(stdout);
  }
#endif /*DEBUG*/

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

void LALApplySearch(LALStatus *status,
		    LALSearchOutput *output,
		    LALSearchInput *input,
		    LALApplySearchParams *params
		    )
{
  INT4 i;
  SearchSummaryInfo info;
  dataBase *dbase;
  CorrelationParamsLW *my=(CorrelationParamsLW *)params->searchParams;

  INITSTATUS( status, "LALApplySearch", CORRELATIONC );
  ATTATCHSTATUSPTR( status );

#ifdef DEBUG
  printf("Search: ncalls=%d\n",my->ncalls);
  fflush(stdout);
#endif /*DEBUG*/

  if(my->ncalls==0){/*compute*/
    my->output=NULL;
    LALCorrelation(status->statusPtr, &my->output, my->input, my->params);
    output->notFinished=1;
    output->fracRemaining=0.5;
    my->ncalls++;
  } else {/*output the results*/
#ifdef DEBUG
    printf("Search: shift=%d\n",(int)(my->output->shift));
    printf("Search: maxCorrelationTimeShift=%d\n",my->output->maxCorrelationTimeShift);
    printf("Search: maxCorrelationValue=%f\n",my->output->maxCorrelationValue);
    printf("Search: minCorrelationTimeShift=%d\n",my->output->minCorrelationTimeShift);
    printf("Search: minCorrelationValue=%f\n",my->output->minCorrelationValue);
    printf("Search: length=%d\n",my->output->length);
    printf("\nSearch: correlations\n\n");
    for(i=0;i<2*my->output->shift+1;i++){
      printf("%f ",my->output->timeShiftedCorrelation[i]);
    }
    printf("\n");
    fflush(stdout);
#endif /*DEBUG*/

    output->numOutput=2;
    output->result=(outPut*)LALCalloc(output->numOutput,sizeof(outPut));

    output->result[0].search=burst;
    output->result[0].significant=1;
    dbase=NULL;

    info.dsoName="correlation";
    info.inputStartTime.gpsSeconds=input->sequences[0].range.dTime.startSec;
    info.inputStartTime.gpsNanoSeconds=input->sequences[0].range.dTime.startNan;
    info.inputEndTime.gpsSeconds=input->sequences[0].range.dTime.stopSec;
    info.inputEndTime.gpsNanoSeconds=input->sequences[0].range.dTime.stopNan;
    info.outputStartTime.gpsSeconds=input->sequences[0].range.dTime.startSec;
    info.outputStartTime.gpsNanoSeconds=input->sequences[0].range.dTime.startNan;
    info.outputEndTime.gpsSeconds=input->sequences[0].range.dTime.stopSec;
    info.outputEndTime.gpsNanoSeconds=input->sequences[0].range.dTime.stopNan;
    info.numEvents=(my->output->maxCorrelationValue > my->threshold);
    info.numNodes=1;
    LALCreateSearchSummaryDataBase(status->statusPtr,&dbase,&info);
    CHECKSTATUSPTR(status);
    output->result[0].results=dbase;

    output->result[0].states=NULL;
    output->result[0].indexNumber=0;

    output->result[0].optional=(multiDimData*)LALMalloc(sizeof(multiDimData));
    if(output->result[0].optional==NULL){
      ABORT(status,CORRELATIONH_ENULL,CORRELATIONH_MSGENULL);
    }
    strcpy(output->result[0].optional->name,
	   "results");
    strcpy(output->result[0].optional->units,"unknown");
    strcpy(output->result[0].optional->comment,"maxCorrelationShift,maxCorrelationValue,minCorrelationShift,minCorrelationValue,shift,length,threshold");
    output->result[0].optional->history=NULL;
    output->result[0].optional->space=timeD;
    output->result[0].optional->type=real_4;
    output->result[0].optional->numberDimensions=1;
    output->result[0].optional->dimensions=(UINT4*)LALMalloc(sizeof(UINT4));
    output->result[0].optional->dimensions[0]=7;
    output->result[0].optional->range.dTime=input->sequences[0].range.dTime;
    output->result[0].optional->data.real4=
      (REAL4*)LALMalloc(sizeof(REAL4)*output->result[0].optional->dimensions[0]);
    output->result[0].optional->data.real4[0]=my->output->maxCorrelationTimeShift;
    output->result[0].optional->data.real4[1]=my->output->maxCorrelationValue;
    output->result[0].optional->data.real4[2]=my->output->minCorrelationTimeShift;
    output->result[0].optional->data.real4[3]=my->output->minCorrelationValue;
    output->result[0].optional->data.real4[4]=(REAL4)my->output->shift;
    output->result[0].optional->data.real4[5]=(REAL4)my->output->length;
    output->result[0].optional->data.real4[6]=my->threshold;

    output->result[1].search=burst;
    output->result[1].significant=1;
    output->result[1].results=NULL;
    output->result[1].states=NULL;
    output->result[1].indexNumber=1;
    output->result[1].optional=(multiDimData*)LALMalloc(sizeof(multiDimData));
    if(output->result[1].optional==NULL){
      ABORT(status,CORRELATIONH_ENULL,CORRELATIONH_MSGENULL);
    }
    strcpy(output->result[1].optional->name,"time shifted correlation");
    strcpy(output->result[1].optional->units,"unknown");
    strcpy(output->result[1].optional->comment,"test");
    output->result[1].optional->history=NULL;
    output->result[1].optional->space=timeD;
    output->result[1].optional->type=real_4;
    output->result[1].optional->numberDimensions=1;
    output->result[1].optional->dimensions=(UINT4*)LALMalloc(sizeof(UINT4));
    output->result[1].optional->dimensions[0]=2*my->output->shift+1;
    output->result[1].optional->range.dTime=input->sequences[0].range.dTime;
    output->result[1].optional->data.real4=(REAL4*)LALMalloc(sizeof(REAL4)*(2*my->output->shift+1));
    for(i=0;i<2*my->output->shift+1;i++){
      output->result[1].optional->data.real4[i]=my->output->timeShiftedCorrelation[i];
    }

    output->notFinished=0;
  }




  DETATCHSTATUSPTR( status );
  RETURN( status );
}

void LALFinalizeSearch(LALStatus *status,
		       void **searchParams
		       )
{
  CorrelationParamsLW *params;

  INITSTATUS( status, "LALFinalizeSearch", CORRELATIONC );

  params=(CorrelationParamsLW *)(*searchParams);

  /* Release memory */

  /* printf("\nAre we here?\n");fflush(stdout); */
  
  LALFree(params->input->one->data);
  LALFree(params->input->two->data);
  LALFree(params->input->one);
  LALFree(params->input->two);
  LALFree(params->params);
  LALFree(params->input);
  LALFree(params->output->timeShiftedCorrelation);
  LALFree(params->output);
  LALFree(params);

  RETURN( status );
}
