/******** <lalVerbatim file="FCTApplySearchCV"> ******** 
Author: Wen, L. 
$Id: FCTApplySearch.c,v 1.1 2001/12/13 00:32:28 teviet Exp $ 
********* </lalVerbatim> ********/ 

#include <lal/Sort.h>
#include <FCTGeneral.h>
#include <FCTApplySearch.h> 
#include <FCTMPI.h>

NRCSID( FCTAPPLYSEARCHC, "$Id: FCTApplySearch.c,v 1.1 2001/12/13 00:32:28 teviet Exp $");

void 
LALApplySearch( 
	       LALStatus *status, 
	       LALSearchOutput *output, 
	       LALSearchInput *input,
	       LALApplySearchParams *applyParams 
	       ) 
{
  INT4  myRank;
  FCTSearchParams *mySearchParams=NULL;
  MPIMessage myMpiMsg; /* initialization message */
  MPI_Comm       myMpiComm; 

  /********************* LAL thing **********************************/ 
  INITSTATUS( status, "LALApplySearch", FCTAPPLYSEARCHC ); 
  ATTATCHSTATUSPTR (status); 
  
  /* check input, output and parameters */
  
  ASSERT (output, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL);
  ASSERT ( !(output->result), status, APPLYSEARCH_ENNUL, APPLYSEARCH_MSGENNUL);

#ifdef FCT_SO_DEBUG
  /*      lalDebugLevel=24; */

#endif
  if ( !(output) ){ 
    ABORT( status, FCTGENERALH_ENULL, FCTGENERALH_MSGENULL);
    
  }

#ifdef FCT_SO_DEBUG  
  printf("in ApplySearch()\n");
#endif
  mySearchParams=(FCTSearchParams *) applyParams->searchParams;

  if ( !mySearchParams )
  {
    ABORT (status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC);
  }

  /******************************************************
	     
	  MPI Initialization for all nodes

  ******************************************************/

  if ( !mySearchParams->numProcs )
  {
    /* allocate memory and check for success   */

    mySearchParams->numProcs = (UINT4 *) LALMalloc( sizeof(UINT4) );
    memset( mySearchParams->numProcs, 0, sizeof(UINT4) );
    
    if( !mySearchParams->numProcs )
    {
      ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
    }
    
    MPI_Comm_size( *(applyParams->mpiParams->comm), mySearchParams->numProcs );
    /* Change in Parallel model *(params->numSlaves) -= 1; */ /*  do it only once for the first call */
    myMpiMsg.send=1;
  }/* endif firstTime */

  /* get the number of search slaves and my Rank (0 for master) */

  MPI_Comm_rank( *(applyParams->mpiParams->comm), &myRank );
  myMpiComm   = *(applyParams->mpiParams->comm);
  myMpiMsg.source = myRank;
  output->numOutput = 0;
  
#ifdef FCT_SO_DEBUG
   fprintf(stdout,"I am node number %i,",myRank);
   fflush( stdout );
#endif

   { 
     INT4                          *startIndices=NULL; 
     INT4                          *stopIndices=NULL; 
     INT4                          dimension;
     INT4                          indexRange;
     INT4                          i;
     INT4 tempStartIndex;     
     dimension=mySearchParams->fctInParams->dimension;
     startIndices =&(mySearchParams->fctInParams->cube->start_locations[0]);
     stopIndices =&(mySearchParams->fctInParams->cube->end_locations[0]);
     
     if (mySearchParams->eventsPerNode == 0) { /* Place level initialization stuff here. */
       
       mySearchParams->eventsPerNode = (REAL4) mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel] /(REAL4) (*(mySearchParams->numProcs));

       mySearchParams->eventStart = mySearchParams->eventsPerNode * myRank;
       mySearchParams->eventStop = mySearchParams->eventsPerNode * (myRank+1);
       if (mySearchParams->curHierLevel == mySearchParams->numHierLevels - 1) {
	 mySearchParams->eventStart = ceil(mySearchParams->eventStart);
	 mySearchParams->eventStop = ceil(mySearchParams->eventStop);
       }
       /*
	 mySearchParams->eventStart = floor( mySearchParams->eventsPerNode * myRank );
	 mySearchParams->eventStop = ceil( mySearchParams->eventsPerNode * (myRank+1) );
	 */
#ifdef FCT_SO_DEBUG  
       printf("eventsPerNode = %f, numProcs = %i, myRank = %i\n"
	      "eventStart = %f, eventStop = %f\n", mySearchParams->eventsPerNode, mySearchParams->numProcs[0], myRank,
	      mySearchParams->eventStart, mySearchParams->eventStop );
#endif       
       if (floor(mySearchParams->eventStart) < mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel]) {
	 
	 /* Setup the stride in the data cube so that we get the correct units. */
	 for (i=1; i<mySearchParams->fctInParams->dimension; i++) {
	   mySearchParams->fctInParams->cube->stride[i]=mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].stride[i];
	 }
#ifdef FCT_SO_DEBUG  
	 printf("Stride set\n");
#endif

	 
	 
	 /* Set the current pointer to the first event that this node will work on. */
	 mySearchParams->curEvent = mySearchParams->hierLevelEvents[mySearchParams->curHierLevel];
	 mySearchParams->curEventNum = 0;
	 if (floor(mySearchParams->eventStart) != mySearchParams->curEventNum) {
	   int myFirstEvent = floor(mySearchParams->eventStart); 
	   for(i = 1; i <= myFirstEvent; i++) {
	     if ( !(mySearchParams->curEvent->nextEvent) ) {
	       ABORT( status, APPLYSEARCH_EVENT, APPLYSEARCH_MSGEVENT );
	     }
	     mySearchParams->curEvent = mySearchParams->curEvent->nextEvent;
	   }
	   mySearchParams->curEventNum = myFirstEvent;
	 }
	 mySearchParams->curEventConditioned = 0;
	 
	 
	 /* In current scheme, each node keeps a list of the N_EVENTS
	    loudest events measured so far.  Initialize this list with
	    non-events (i.e. events with no power). */
	 mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1] = createHierEvent( mySearchParams->fctInParams->dimension );
	 if ( !(mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1]) ) {
	   ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
	 } else {
	   FCTHierEventType *thisEvent = mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1];
	   memcpy( thisEvent->location, mySearchParams->hierLevelEvents[mySearchParams->curHierLevel]->location,
		   mySearchParams->fctInParams->dimension*sizeof(REAL4) );
	   thisEvent->location[0] = 0.5*mySearchParams->inputDataStepSize*mySearchParams->inputDataLength;
	   thisEvent->power = 0.0;
	   thisEvent->nextEvent = NULL;
#ifdef FCT_SO_DEBUG
	   printf( "Created event 0 at level %i, location[1]=%10.3e\n", mySearchParams->curHierLevel+1, thisEvent->location[1] );
#endif
	   for ( i = 1; i < N_EVENTS; i++ ) {
	     if ( !( thisEvent->nextEvent = createHierEvent( mySearchParams->fctInParams->dimension ) ) ) {
	       ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
	     }
	     thisEvent = thisEvent->nextEvent;
	     memcpy( thisEvent->location, mySearchParams->hierLevelEvents[mySearchParams->curHierLevel]->location,
		     mySearchParams->fctInParams->dimension*sizeof(REAL4) );
	     thisEvent->location[0] = 0.5*mySearchParams->inputDataStepSize*mySearchParams->inputDataLength;
	     thisEvent->power = 0.0;
	     thisEvent->nextEvent = NULL;
#ifdef FCT_SO_DEBUG
	     printf( "Created event %i at level %i, location[1]=%10.3e\n", i, mySearchParams->curHierLevel+1, thisEvent->location[1] );
#endif
	   }
	   mySearchParams->lowestEvent[mySearchParams->curHierLevel+1] = 0;
	   mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel+1] = N_EVENTS;
	 }

	 /* Put threshold calculation here. */
	 mySearchParams->powerThreshold[mySearchParams->curHierLevel] = MIN_THRESHOLD;
	 
       } else {
	 mySearchParams->curEvent = 0;
	 mySearchParams->curEventNum = mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel];
       }	 

#ifdef FCT_SO_DEBUG
       printf("Done setting up stuff for hierLevel %i.\n",mySearchParams->curHierLevel);
#endif
     }      	

 
     if (!mySearchParams->curEventConditioned && (mySearchParams->curEventNum < mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel])) {
       INT4 rowsInVol = 1;
       if (mySearchParams->curHierLevel != 0) {
	 HierConditionData(status->statusPtr, input, mySearchParams);
       } else {
	 mySearchParams->curEventConditioned = 1;
       }
       
       /* Move the volume counter to where we're going to start. */
#ifdef FCT_SO_DEBUG
       printf("Moving to first subvolume of event.\n");
#endif
       for (i = 1; i < mySearchParams->fctInParams->dimension; i++) {
	 startIndices[i] = (INT4)((mySearchParams->curEvent->location[i] - \
				   mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].range[i]/2.0) 
				  / mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].fctUnits[i]);
	 /*stopIndices[i] = (INT4)((mySearchParams->curEvent->location[i] + \
	   mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].range[i]/2.0) 
	   / mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].fctUnits[i]);
	   */
	 stopIndices[i] = startIndices[i] + mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[i];
#ifdef FCT_SO_DEBUG
	 printf("location[%i]=%10.3e range[%i]=%10.3e units[%i]=%10.3e\nstart[%i]=%i, stop[%i]=%i\n\n", i,
		mySearchParams->curEvent->location[i], i,
		mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].range[i], i,
		mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].fctUnits[i], i,
		startIndices[i], i, stopIndices[i] );
#endif
       }
#ifdef FCT_SO_DEBUG
       printf("At Initial subvolume of event.\n");

       printf("Reducing Parallel cut dimension.\n");
#endif
       
       i = mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].parallelCutDimension;
       tempStartIndex = startIndices[i] + floor(((REAL4) mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[i]) *
						(mySearchParams->eventStart - floor(mySearchParams->eventStart)));
       if ( floor(mySearchParams->eventStop) < floor(mySearchParams->eventStart) + 1 ) {
	 stopIndices[i] = startIndices[i] + floor(((REAL4) indexRange) * (mySearchParams->eventStop - floor(mySearchParams->eventStop)));
       }
       startIndices[i] = tempStartIndex;
       
#ifdef FCT_SO_DEBUG
       printf("parallel cut dimension reduced to:\n");
       printf("  eventStart=%f, eventStop=%f\n", mySearchParams->eventStart, mySearchParams->eventStop );
       printf("  start[%i]=%i, stop[%i]=%i\n", i, startIndices[i], i, stopIndices[i] );
#endif
       
       for (i = 1; i < mySearchParams->fctInParams->dimension; i++){
	 rowsInVol *= (stopIndices[i] - startIndices[i] + 1);
       }
       
#ifdef FCT_SO_DEBUG
       printf("mySearchParams->fctInParams->rowsPerApplySearch = %i\n", mySearchParams->fctInParams->rowsPerApplySearch );
       printf("Calculating the number of report slices.\n");
#endif
       i = mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].reportCutDimension;
       mySearchParams->numReportSlices = rowsInVol / mySearchParams->fctInParams->rowsPerApplySearch + 1;
       if ( ( mySearchParams->numReportSlices )*( *mySearchParams->numProcs ) < 10 )
	 mySearchParams->numReportSlices = 10/( *mySearchParams->numProcs ) + 1;

#ifdef FCT_SO_DEBUG
       printf("rowsInVol = %i\n", rowsInVol );
       printf("mySearchParams->numReportSlices = %i\n", mySearchParams->numReportSlices );
#endif
       
       mySearchParams->curReportSlice = 0;
       stopIndices[i] = startIndices[i] +   ceil((REAL4) mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[i] / (REAL4)mySearchParams->numReportSlices);
     }
     
   
     
     /* Move to bottom */ 
     /*i = mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].reportCutDimension;
       indexRange = stopIndices[i] - startIndices[i] + 1;
       startIndices[i] = stopIndices[i] + 1;
       stopIndices[i] = startIndices[i] + indexRange / mySearchParams.numReportSlices - 1; 
       mySearchParams.curReportSlices++; */
     
     
     /**************************************************************
       
       Call FCT 
       
       ***************************************************************/	
     /*  FCT Search Engine -Linqing; need to make sure that the segnum != 0
	 for (i = 0; i < mySearchParams->fctInParams->dimension; i++)
	 {
	 
	 mySearchParams->fctInParams->cube->start_locations[i] *=LambMax;
	 mySearchParams->fctInParams->cube->end_locations[i] *=LambMax;
	 }  */

     if ( mySearchParams->curEventNum < mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel]) {
       LALMyFCTInterf(status->statusPtr, &mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1], mySearchParams);
       /* FCT */
       CHECKSTATUSPTR (status); 
     }
     
     output->fracRemaining = 1 - 1.0 * (((REAL4)mySearchParams->curHierLevel + 1.0)/(REAL4)mySearchParams->numHierLevels) * \
       ((REAL4) mySearchParams->curEventNum - (REAL4) floor(mySearchParams->eventStart) + 1.0)/(REAL4) ceil(mySearchParams->eventsPerNode) * \
       ((REAL4) mySearchParams->curReportSlice + 1.0)/(REAL4)mySearchParams->numReportSlices; 
     output->notFinished = 1; 
#ifdef FCT_SO_DEBUG  
     printf("curEventNum = %d fracRemaining = %f\n", mySearchParams->curEventNum, output->fracRemaining );
#endif
     
     if (mySearchParams->curReportSlice < mySearchParams->numReportSlices - 1) {
       /* We can do more report slices in this event, so let's advance the report slice. */
       mySearchParams->curReportSlice++;
#ifdef FCT_SO_DEBUG  
       printf("mySearchParams->curReportSlice = %i\n", mySearchParams->curReportSlice );
       fflush( stdout );
#endif
       
       /* We can leave the indices alone in all the dimensions other than the reportCutDimension */
       i = mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].reportCutDimension;
       tempStartIndex = (INT4)((mySearchParams->curEvent->location[i] - \
				 mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].range[i]/2.0) 
				/ mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].fctUnits[i]);
       startIndices[i] = tempStartIndex +  ceil((REAL4) mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[i] * (REAL4) mySearchParams->curReportSlice / (REAL4) mySearchParams->numReportSlices);
       stopIndices[i] = tempStartIndex +   ceil((REAL4) mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[i] * (REAL4) (mySearchParams->curReportSlice + 1) / (REAL4) mySearchParams->numReportSlices);
     } else if (mySearchParams->curEventNum + 1 < floor(mySearchParams->eventStop)) {
       /* Let's copy the event to the next hierarchy level so that we can fake multiple levels.
       FCTHierEventType *lastEvent;
#ifdef FCT_SO_DEBUG  
       fprintf(stdout,"copying the event to the next hierarchy level so that we can fake multiple levels.\n" );
       fflush( stdout );
#endif

       if ( mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel+1] < 5 ) {
	 lastEvent = mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1];
	 if (lastEvent) {
	   while ( lastEvent->nextEvent ) {
	     lastEvent = lastEvent->nextEvent;
	   }
	   lastEvent->nextEvent = createHierEvent( mySearchParams->fctInParams->dimension );
	   lastEvent = lastEvent->nextEvent;
	 } else {
	   lastEvent = mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1] = 
	   createHierEvent( mySearchParams->fctInParams->dimension );
	 }
	 lastEvent->nextEvent = NULL;
	 
	 lastEvent->location[0] = 250.0 + mySearchParams->inputDataStart;
	 for (i = 1; i < mySearchParams->fctInParams->dimension; i++) {
	   lastEvent->location[i] = mySearchParams->curEvent->location[i];
	 }
	 mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel+1]++;
       }
#ifdef FCT_SO_DEBUG  
       fprintf(stdout,"Done copying event.\n" );
       fflush( stdout );
#endif*/

       /* We can still advance the events. */
       mySearchParams->curEvent = mySearchParams->curEvent->nextEvent;
       mySearchParams->curEventNum++;
       mySearchParams->curEventConditioned = 0;
     } else {
       /* We're done with this level. */
       if (mySearchParams->curHierLevel < mySearchParams->numHierLevels - 1) {
	 /* There are more levels to process.  Share the event list and move to the next level. */
#ifdef FCT_SO_DEBUG  
	 printf("Done with level moving to next\n");
	 fflush( stdout );
#endif
	 ShareEventList(status->statusPtr, mySearchParams, myMpiComm);
	 CHECKSTATUSPTR (status);

	 mySearchParams->eventsPerNode = 0;
	 mySearchParams->curHierLevel++;
       } else {
	 /* This is the last level. Build the output. */
	 /*TRANSLATION FUNCTION */
	 TranslateEventListAndBuildOutput (status->statusPtr, output, mySearchParams);
	 /*LALBuildFCTOutput (status->statusPtr, output, params->fctInParams->numEvents, event);*/
#ifdef FCT_SO_DEBUG  
	 printf("Done with the Search.\n");
	 fflush( stdout );
#endif

	 CHECKSTATUSPTR (status);       
	 output->fracRemaining = 0; 
	 output->notFinished = 0; 
       }
     }
   }
#ifdef FCT_SO_DEBUG
   printf(" out of ApplySearch ()\n");
#endif

   DETATCHSTATUSPTR (status); 
   RETURN( status ); 

} /* end applySearch() */

void ShareEventList(LALStatus *status,  FCTSearchParams * mySearchParams, MPI_Comm comm) {
  REAL4 *myEventList;
  REAL4 *totalEventList;
  INT4  realsPerEvent;
  INT4  *bytesOnNode;
  INT4  *realsOnNode;
  INT4  *displs;
  INT4  i;
  INT4  realsOffset;
  INT4  realsOnThisNode;
  INT4  totalReals;
  INT4  totalEvents;
  INT4Vector *idx = NULL;
  REAL4Vector *power = NULL;

  INITSTATUS( status, "ShareEventList", FCTAPPLYSEARCHC );
  ATTATCHSTATUSPTR (status);

  /* It might be possible to use only the totalEventList instead of myEventList and save some memory.*/

  /* Currently the event structure contains only REAL4 location[dimension] and REAL4 power. They will
     be stored in that order. */
  realsPerEvent = mySearchParams->fctInParams->dimension + 1; /* plus 1 for the power */

  realsOnThisNode = realsPerEvent*mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel+1];
#ifdef FCT_SO_DEBUG  
  printf("realsOnThisNode = %i, realsPerEvent = %i, numEvents = %i\n", realsOnThisNode, realsPerEvent,
	 mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel+1] );
#endif
  myEventList = (REAL4*) LALMalloc(sizeof(REAL4)*realsOnThisNode);
  realsOnNode = (INT4 *) LALMalloc(sizeof(INT4)*(*mySearchParams->numProcs));
  bytesOnNode = (INT4 *) LALMalloc(sizeof(INT4)*(*mySearchParams->numProcs));
  displs = (INT4 *) LALMalloc(sizeof(INT4)*(*mySearchParams->numProcs));

  if (!myEventList || !realsOnNode || !displs) {
    if (myEventList)
      LALFree(myEventList);
    if (realsOnNode)
      LALFree(realsOnNode);
    if (displs)
      LALFree(displs);
    ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
  }

  mySearchParams->curEvent = mySearchParams->hierLevelEvents[mySearchParams->curHierLevel + 1];
  for (mySearchParams->curEventNum = 0;
       mySearchParams->curEventNum < mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel + 1];
       mySearchParams->curEventNum++){
    if (!mySearchParams->curEvent) {
      ABORT( status, APPLYSEARCH_EVENT, APPLYSEARCH_MSGEVENT );
    }
    realsOffset = mySearchParams->curEventNum * realsPerEvent;
    for (i = 0; i < mySearchParams->fctInParams->dimension; i++){
      myEventList[realsOffset+i] = mySearchParams->curEvent->location[i];
    }
    myEventList[realsOffset+mySearchParams->fctInParams->dimension] = mySearchParams->curEvent->power;
    mySearchParams->curEvent = mySearchParams->curEvent->nextEvent;
  }
  destroyEventList(mySearchParams->hierLevelEvents[mySearchParams->curHierLevel + 1]);
  mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel + 1] = 0;

#ifdef FCT_SO_DEBUG
  fprintf(stdout,"Doing the MPI_Allgather\n" );
  fflush( stdout );
#endif
  /* It's a little annoying, but because we don't know what datatype REAL4 is or INT4, we have to use MPI_BYTE. */
  MPI_Allgather(&realsOnThisNode, sizeof(realsOnThisNode), MPI_BYTE, realsOnNode, sizeof(realsOnNode[0]), MPI_BYTE, comm);
#ifdef FCT_SO_DEBUG
  fprintf(stdout,"Done with the MPI_Allgather\n" );
  fflush( stdout );
#endif

  bytesOnNode[0] = sizeof(myEventList[0]) * realsOnNode[0];
  displs[0] = 0;
  totalReals = realsOnNode[0];
  for (i = 1; i < (*mySearchParams->numProcs); i++){
    bytesOnNode[i] = sizeof(myEventList[0]) * realsOnNode[i];
    displs[i] = displs[i-1] + bytesOnNode[i-1];
    totalReals += realsOnNode[i];
  }
  totalEvents = totalReals / realsPerEvent;

  totalEventList = (REAL4*) LALMalloc(sizeof(REAL4)*totalReals);
  if (!totalEventList) {
    LALFree(myEventList);
    LALFree(realsOnNode);
    LALFree(bytesOnNode);
    LALFree(displs);
    ABORT( status, APPLYSEARCH_EVENT, APPLYSEARCH_MSGEVENT );
  }
#ifdef FCT_SO_DEBUG
  fprintf(stdout,"Doing the MPI_Allgatherv\n" );
  fflush( stdout );
#endif
  MPI_Allgatherv(myEventList, realsOnThisNode*sizeof(myEventList[0]), MPI_BYTE, totalEventList, bytesOnNode, displs, MPI_BYTE, comm);
  LALFree(myEventList);
  LALFree(realsOnNode);
  LALFree(bytesOnNode);
  LALFree(displs);

#ifdef FCT_SO_DEBUG
  fprintf(stdout,"Done with the MPI_Allgatherv\n" );
  fflush( stdout );
#endif

  /* Construct array indexing events from weakest to strongest. */
  LALSCreateVector( status->statusPtr, &power, totalEvents );
  BEGINFAIL( status )
    LALFree(totalEventList);
  ENDFAIL( status );
  LALI4CreateVector( status->statusPtr, &idx, totalEvents );
  BEGINFAIL( status ) {
    TRY( LALSDestroyVector( status->statusPtr, &power ), status );
    LALFree(totalEventList);
  } ENDFAIL( status );
  for ( i = 0; i < totalEvents; i++ )
    power->data[i] = -totalEventList[ realsPerEvent*i + mySearchParams->fctInParams->dimension ];
  LALSHeapIndex( status->statusPtr, idx, power );
  BEGINFAIL( status ) {
    TRY( LALSDestroyVector( status->statusPtr, &power ), status );
    TRY( LALI4DestroyVector( status->statusPtr, &idx ), status );
    LALFree(totalEventList);
  } ENDFAIL( status );
  TRY( LALSDestroyVector( status->statusPtr, &power ), status );

  /* Create linked list of the N_EVENTS loudest events, or all given events with positive power if there are fewer. */
  realsOffset = idx->data[0]*realsPerEvent;
  mySearchParams->curEvent = mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1]
    = createHierEvent( mySearchParams->fctInParams->dimension );
  if ( !mySearchParams->curEvent ) {
    TRY( LALI4DestroyVector( status->statusPtr, &idx ), status );
    destroyEventList( mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1] );
    ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
  }
  memcpy( mySearchParams->curEvent->location, totalEventList + realsOffset, mySearchParams->fctInParams->dimension*sizeof(REAL4) );
  mySearchParams->curEvent->power = totalEventList[realsOffset+mySearchParams->fctInParams->dimension];
  mySearchParams->curEvent->nextEvent = NULL;
  mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel+1] = 1;
  for ( i = 1; i < N_EVENTS && i < totalEvents; i++ ) {
    realsOffset = idx->data[i]*realsPerEvent;
    if ( totalEventList[realsOffset+mySearchParams->fctInParams->dimension] <= 0.0 ) {
      i = N_EVENTS;
    } else {
      mySearchParams->curEvent->nextEvent = createHierEvent( mySearchParams->fctInParams->dimension );
      mySearchParams->curEvent = mySearchParams->curEvent->nextEvent;
      if ( !mySearchParams->curEvent ) {
	TRY( LALI4DestroyVector( status->statusPtr, &idx ), status );
	destroyEventList( mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1] );
	ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
      }
      memcpy( mySearchParams->curEvent->location, totalEventList + realsOffset, mySearchParams->fctInParams->dimension*sizeof(REAL4) );
      mySearchParams->curEvent->power = totalEventList[realsOffset+mySearchParams->fctInParams->dimension];
      mySearchParams->curEvent->nextEvent = NULL;
      mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel+1]++;
    }
  }

  /*
  for (mySearchParams->curEventNum = 0; mySearchParams->curEventNum < totalEvents; mySearchParams->curEventNum++){
    if (mySearchParams->curEventNum == 0) {
      mySearchParams->curEvent = mySearchParams->hierLevelEvents[mySearchParams->curHierLevel + 1] = createHierEvent(mySearchParams->fctInParams->dimension);
    } else {
      mySearchParams->curEvent->nextEvent = createHierEvent(mySearchParams->fctInParams->dimension);
      mySearchParams->curEvent = mySearchParams->curEvent->nextEvent;
    }
    if (!mySearchParams->curEvent) {
      ABORT()
    }   
    realsOffset = mySearchParams->curEventNum * realsPerEvent;
    for (i = 0; i < mySearchParams->fctInParams->dimension; i++){
      mySearchParams->curEvent->location[i] = totalEventList[realsOffset+i];
    }
    mySearchParams->curEvent->power = totalEventList[realsOffset+mySearchParams->fctInParams->dimension];
  } 
  mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel+1] = totalEvents;
  */
#ifdef FCT_SO_DEBUG
  printf( "Total number of events = %i\n", totalEvents );
  fprintf(stdout,"Leaving ShareEventList\n" );
  fflush( stdout );
#endif

  TRY( LALI4DestroyVector( status->statusPtr, &idx ), status );
  LALFree(totalEventList);
  DETATCHSTATUSPTR (status);
  RETURN( status );
}


