/*----------------------------------------------------------------------- 
 * 
 * File Name: BuildOutput.c
 *
 * Author: Brown, D. A. and Wen, L.
 * 
 * $Id: BuildOutput.c,v 1.10 2001/12/28 01:31:44 lwen Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include <lal/LALDatatypes.h>
#include <FCTGeneral.h>
 

NRCSID( BUILDOUTPUTC, "$Id: BuildOutput.c,v 1.10 2001/12/28 01:31:44 lwen Exp $" );

void
LALBuildFCTOutput (
    LALStatus          *status,
    LALSearchOutput    *output,
    INT4                numEvents,
    FCTEventType       *eventList,
    CHAR               *ifoName
                    )
{
  outPut               *outputArray;
  FCTEventType         *event;
  dataBase             *prev;
  dataBase             *dbEntry;
  UINT4                 i;


  INITSTATUS (status, "LALBuildFCTOutput", BUILDOUTPUTC);


  /*
   *
   * print the list events as recieved by the master
   *
   */

  /*
#ifdef FCT_SO_DEBUG

  event = eventList;

  while ( event )
  {
    fprintf( stdout, "build output: event id = %d\n", 
        event->id );
    fprintf( stdout, "build output: time.gpsSeconds = %d\n", 
        event->time.gpsSeconds );
    fprintf( stdout, "build output: time.gpsNanoSeconds = %d\n", 
        event->time.gpsNanoSeconds );

    fprintf( stdout, "build output: mass1 = %f\n", 
        event->mass1 );
    fprintf( stdout, "build output: mass2 = %f\n", 
        event->mass2 );

    fprintf( stdout, "build output: snr = %e\n", 
        event->snr );
    fprintf( stdout, "build output: chisq = %e\n", 
        event->chisq );
    fprintf( stdout, "build output: sigma = %e\n", 
        event->sigma );

    fprintf( stdout, "build output: effDist = %e\n", 
        event->effDist );

    fflush( stdout );

    event = event->nextEvent;
  }
#endif
  */ 

  /*
   *
   * count the events and create memory for the outPut array
   *
   */

  output->numOutput = numEvents; /* assigned in FCT_LAL.c */
 
#ifdef FCT_SO_DEBUG
  fprintf( stdout, "build output: there are %d events in the list\n", 
     numEvents  );
  fflush( stdout );
#endif

  outputArray = output->result = (outPut *)    LALMalloc( numEvents * sizeof(outPut) );


  /*
   *
   * fill the array of output
   *
   */


  for ( i = 0, event = eventList; (i < numEvents) && event; ++i, event = event->nextEvent )
  {
    /* this is the same for every output */
    outputArray[i].search         = binaryInspiral;
    outputArray[i].significant    = 1;
    outputArray[i].states         = NULL;
    outputArray[i].optional       = NULL;

    /* template index number */
    outputArray[i].indexNumber =0; /* event->tmplt.number; */


    /* 
     *
     * alloc memory for the database linked list
     *
     */


    /* filter id */
    /*
     * dbEntry = outputArray[i].results = 
     *  (dataBase *) LALMalloc( sizeof(dataBase) );
     * dbEntry->previous = NULL;
     * 
     * sprintf( dbEntry->tableName, "sngl_inspiral" );
     * sprintf( dbEntry->columnName, "filter_id" );
     * 
     * dbEntry->type       = char_s_ptr;
     * dbEntry->numberRows = 1;
     */

    /* scope the blobStr variable... */
    /*
     * {
     *  UINT4 j = 0;
     *  const CHAR blobStr[] = "inspiral";
     *
     *  dbEntry->rows.chars = (CHAR *) 
     *    LALMalloc( strlen( blobStr ) * sizeof(CHAR) );
     *  while ( blobStr[j] ) dbEntry->rows.chars[j++] = blobStr[j];
     *
     *  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     *  *(dbEntry->rowDimensions) = (UINT4) strlen( blobStr );
     * }
     * 
     * prev = dbEntry;
     */

    /* ifo name */
    /*
     * dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     * dbEntry->previous = prev;
     */
    dbEntry = outputArray[i].results = 
      (dataBase *) LALMalloc( sizeof(dataBase) );
    /* check for memory allocation */
    
    dbEntry->previous = NULL;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "ifo" );

    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;

    dbEntry->rows.chars = 
      (CHAR *) LALMalloc( 2 * dbEntry->numberRows * sizeof(CHAR) );
    dbEntry->rows.chars[0] = ifoName[0];
    dbEntry->rows.chars[1] = ifoName[1]; /* what is this? */

    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = 2;

    prev = dbEntry;

    /* search */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "search" );

    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;

    /* scope the blobStr variable... */
    {
       UINT4 j = 0;
      const CHAR blobStr[] = "FCT";

      dbEntry->rows.chars = (CHAR *) 
        LALMalloc( strlen( blobStr ) * sizeof(CHAR) );
      while ( blobStr[j]) dbEntry->rows.chars[j++] = blobStr[j];

      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = (UINT4) strlen( blobStr );
    }

    prev = dbEntry;

    /* end time seconds */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "end_time" );

    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = floor(event->tC);

    prev = dbEntry;

    /* end time nanoseconds */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "end_time_ns" );

    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = ceil(1.0e9*(event->tC-floor(event->tC)));

    prev = dbEntry;

    /* impulse time seconds */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "impulse_time" );

    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = event->time.gpsSeconds;

    prev = dbEntry;

    /* impulse time nanoseconds */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "impulse_time_ns" );

    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = event->time.gpsNanoSeconds;

    prev = dbEntry;
    
    /* amplitude (dummy value) */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "amplitude" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = 0.0;

    prev = dbEntry;

    /* effective distance */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "eff_distance" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = event->effDist;

    prev = dbEntry;

    /* coalesence phase */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "coa_phase" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = 0.0;

    prev = dbEntry;

    /* m1 */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "mass1" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = (REAL4) event->mass1;

    prev = dbEntry;

    /* m2 */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "mass2" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = (REAL4) event->mass2;

    prev = dbEntry;

    /* mchirp */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "mchirp" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    {
      REAL4 m1 = (REAL4) event->mass1;
      REAL4 m2 = (REAL4) event->mass2;
      REAL4 eta = (REAL4) event->eta;
      *(dbEntry->rows.real4) = (m1 + m2) * pow( eta, 3.0/5.0 ) ;
    }

    prev = dbEntry;

    /* eta */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "eta" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = (REAL4) event->eta;

    prev = dbEntry;

    /* tau0 */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "tau0" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = (REAL4)event->t0;

    prev = dbEntry;

    /* tau2 */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "tau2" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = (REAL4)event->t2;

    prev = dbEntry;
    
    /* tau3 */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "tau3" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = (REAL4)event->t3;

    prev = dbEntry;
    
    /* tau4 */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "tau4" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = (REAL4)event->t4;

    prev = dbEntry;
    
    /* tau5 */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "tau5" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) =0.0;

    prev = dbEntry;
    
    /* ttotal */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "ttotal" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = 0.0;

    prev = dbEntry;

    /* snr */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "snr" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = event->snr;

    prev = dbEntry;

    /* chisq */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "chisq" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = event->chisq;

    prev = dbEntry;

    /* chisq degress of freedom (dummy) */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "chisq_dof" );

    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = 0;

    prev = dbEntry;

    /* sigma squared */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;;

    sprintf( dbEntry->tableName, "sngl_inspiral" );
    sprintf( dbEntry->columnName, "sigmasq" );

    dbEntry->type       = real_8;
    dbEntry->numberRows = 1;
    dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real8) = event->sigma * event->sigma;

    dbEntry->next = NULL;

  }


  RETURN (status);
}

void
BuildFilterOutPut (
    LALStatus          *status,
    LALSearchOutput    *output,
    REAL4Vector        *rhosqVec
                    )
{
  outPut               *outputArray;
  multiDimData         *mddEntry;


  INITSTATUS (status, "BuildFilterOutPut", BUILDOUTPUTC);

  /*
   *
   * create memory for the outPut array
   *
   */


  outputArray = output->result = (outPut *)
    LALMalloc( sizeof(outPut) );
  memset( outputArray, 0, sizeof(outPut) );

  output->numOutput = 1;


  /*
   *
   * fill the array of output
   *
   */


  /* this is the same for every output */
  outputArray->indexNumber = 0;
  outputArray->search         = binaryInspiral;
  outputArray->significant    = 1;
  outputArray->states         = NULL;
  outputArray->results        = NULL;


  /* 
   *
   * alloc memory for the optional linked list
   *
   */

  
  /* allocate memory */
  mddEntry = outputArray->optional = 
    (multiDimData *) LALMalloc( sizeof(multiDimData) );
  memset( mddEntry, 0, sizeof(multiDimData) );

  /* write the name and domain of the data */
  sprintf( mddEntry->name, "filter_output" );
  sprintf( mddEntry->units, "dimensionless" );
  mddEntry->space = timeD;

  /* data type is REAL4 */
  mddEntry->type = real_4;
  
  /* set the interval union to zero */
  memset( &(mddEntry->range.dTime), 0, sizeof(gpsTimeInterval) );

  /* size of the data */
  mddEntry->numberDimensions = 1;
  mddEntry->dimensions       = (UINT4 *) LALMalloc( sizeof(UINT4) );
  mddEntry->dimensions[0]    = rhosqVec->length;

  /* dcHistory is meaningless here */
  mddEntry->history = NULL;

  /* allocate memory for the data */
  mddEntry->data.real4 = (REAL4 *) 
    LALCalloc( rhosqVec->length, sizeof(REAL4) );

  /* and copy the data there */
  memcpy( mddEntry->data.real4, rhosqVec->data, 
      rhosqVec->length * sizeof(REAL4) );

  /* and hope for the best */
  RETURN (status);
}
