/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="SlopeMakeEventTableC"> ********
Author: Daw, E
$Id: SlopeMakeEventTable.c,v 1.3 2003/01/21 23:55:39 edaw Exp $
******** </lalVerbatim> ********/

#ifdef DEBUG_SLOPE
#include <stdio.h>
#endif
#include <math.h>

/* lal standard library */
#include <lal/LALStdlib.h>

/* lal MPI interface */
#include <lal/Comm.h>

/* structures and definitions for this code segment */
#include <SlopeDatatypes.h>
#include <SlopeAddTimeToGpsSecNan.h>
#include <SlopeAddEventToLinkedList.h>
#include <SlopeApplySearchMaster.h>

NRCSID( SLOPEMAKEEVENTTABLEC, "$Id: ");

void
SlopeMakeEventTable(LALStatus* status,
		    SlopeSearchParams* slopeparams,
		    LALSearchOutput* output) {
  /* program name appearing in program field of SLOPE_FILTER table */

  /*this field used in Patrick Brady's moments code, not in power*/
  /*const CHAR* const PROGRAM = "slope";*/

  /* pointer to an output structure */
  outPut* poutput = 0x0;
  /* pointers to a dataBase structure */
  dataBase* db = 0x0;
  dataBase* dbprevious = 0x0;
  /* counter for events in the event list */
  UINT4 eventCounter = 0;
  /* pointer to an event list element structure */
  SlopeDetections* eventList;
  /* counter for colums in the database table */
  UINT4 dbCC=0;
  /* number of columns in database table */
  UINT4 database_column_count = 10;
  UINT4 addSummary=0;

  INITSTATUS( status, "MakeEventTable", SLOPEMAKEEVENTTABLEC );
  ATTATCHSTATUSPTR(status);

  /* set number of events to number of peaks found */
  output->numOutput = slopeparams->numberOfTriggersInThisSearch;
  /* allocate memory for database entries for each trigger */

  if(slopeparams->summary!=NULL){
    addSummary=1;
  }

#ifdef SLDEBUG
  printf("making output array for %u outputs \n",slopeparams->numberOfTriggersInThisSearch);
#endif
  poutput = LALCalloc( slopeparams->numberOfTriggersInThisSearch + addSummary, sizeof( outPut ) ); 
  output->result = poutput;
#ifdef SLDEBUG
  printf("made output array\n");
#endif

  /* loop over events, filling the outPut structure for each event */
  eventCounter = 0;
  eventList = slopeparams->firstEvent;

  while( eventCounter < (slopeparams->numberOfTriggersInThisSearch)) {

#ifdef SLDEBUG
    printf("Writing event %u.\n",eventCounter);
#endif

    (poutput+eventCounter)->search           = burst;
    (poutput+eventCounter)->significant      = 1;
    (poutput+eventCounter)->states           = NULL;
    (poutput+eventCounter)->optional         = NULL;
    (poutput+eventCounter)->indexNumber      = 0;
    (poutput+eventCounter)->results = LALMalloc( sizeof(dataBase) );
    db = (poutput+eventCounter)->results;
    /* code to make a single line of the database goes here */ 
    for(dbCC = 0; dbCC<database_column_count; ++dbCC) {
      if(dbCC == 0) {
	db->previous = NULL;
      } else {
	db->previous = dbprevious;
      }
      sprintf(db->tableName,"sngl_burst");
      db->numberRows = 1;
      db->rowDimensions = NULL;
      switch (dbCC) {
      case 0:
	sprintf(db->columnName,"start_time");
	db->type = int_4s;
	db->rows.int4s = (INT4*) LALMalloc(sizeof(INT4));
	*(db->rows.int4s) = (INT4) eventList->eventDetectionTimeSeconds;
	break;
      case 1:
	sprintf(db->columnName,"start_time_ns");
	db->type = int_4s;
	db->rows.int4s = (INT4*) LALMalloc(sizeof(INT4));
	*(db->rows.int4s) = (INT4) eventList->eventDetectionTimeNanoseconds;
	break;
      case 2:
	sprintf(db->columnName,"duration");
	db->type = real_4;
	db->rows.real4 = (REAL4*) LALMalloc(sizeof(REAL4));
	*(db->rows.real4) = eventList->eventDuration;
	break;
      case 3:
	sprintf(db->columnName,"central_freq");
	db->type = real_4;
	db->rows.real4 = (REAL4*) LALMalloc(sizeof(REAL4));
	*(db->rows.real4) = eventList->eventCentralFrequency;
	break;
      case 4:
	sprintf(db->columnName,"bandwidth");
	db->type = real_4;
	db->rows.real4 = (REAL4*) LALMalloc(sizeof(REAL4));
	*(db->rows.real4) = eventList->eventBandwidth;
	break;
      case 5:
	sprintf(db->columnName,"amplitude");
	db->type = real_4;
	db->rows.real4 = (REAL4*) LALMalloc(sizeof(REAL4));
	*(db->rows.real4) = eventList->eventAmplitude;
	break;
      case 6:
	sprintf(db->columnName,"confidence");
	db->type = real_4;
	db->rows.real4 = (REAL4*) LALMalloc(sizeof(REAL4));
	*(db->rows.real4) = 0.9;
	break;
      case 7:
	sprintf(db->columnName,"search");
	db->type = char_s_ptr;
        db->rows.chars = (CHAR*) LALMalloc(6*sizeof(CHAR));
	db->rowDimensions = LALMalloc(sizeof(UINT4));
	*(db->rowDimensions) = 5;
	strcpy(db->rows.chars , "slope");
	break;
      case 8:
	sprintf(db->columnName,"ifo");
	db->type = char_s_ptr;
        db->rows.chars = (CHAR*) LALMalloc(3*sizeof(CHAR));
	db->rowDimensions = LALMalloc(sizeof(UINT4));
	*(db->rowDimensions) = 2;
	strcpy(db->rows.chars , slopeparams->ifoName);
	break;
      case 9:
        sprintf(db->columnName,"snr");
        db->type = real_4;
	db->rows.real4 = (REAL4*) LALMalloc(sizeof(REAL4));
        *(db->rows.real4) = eventList->eventSnr;
	break;
      }
      if( dbCC == (database_column_count - 1)) {
	db->next = NULL;
      } else {
	db->next = LALMalloc( sizeof(dataBase) );
	dbprevious = db;
	db = db->next;
      }
    }
    ++eventCounter;
    eventList = (eventList->next);
#ifdef SLDEBUG
    printf("%#x\n",(unsigned int)eventList);
#endif

  }

  if(slopeparams->summary!=NULL){
    (poutput+eventCounter)->search           = burst;
    (poutput+eventCounter)->significant      = 1;
    (poutput+eventCounter)->states           = NULL;
    (poutput+eventCounter)->optional         = NULL;
    (poutput+eventCounter)->indexNumber      = 0;
    (poutput+eventCounter)->results = slopeparams->summary;
    output->numOutput++;
  }

  DETATCHSTATUSPTR(status);
  RETURN(status);
}
