/*************** <lalVerbatim file="InitCV"> ****************
Author: Klimenko, S. and Yakushin, I.
$Id: Init.c,v 1.1 2003/01/02 23:20:34 jolien Exp $
**************** </lalVerbatim> *************************************/
#include <stdio.h>
#include <string.h>
#include <lal/LALStdlib.h>
#include <lal/Units.h>
#include <Wave.h>
#include <BuildDB.h>

static void _defaults(WaveParams *params);


NRCSID(INITC, "$Id" );

static void _defaults(WaveParams *params)
{
  params->nonZeroFraction=0.3;
  params->usePixelMixer=FALSE;
  params->seed=time(NULL);
  params->usePixelSwapOne=FALSE;
  params->usePixelSwapTwo=FALSE;
  params->timeWindowNanoSec=0;
  params->aura=FALSE;
  params->minClusterSize=3;
  params->maxClusterSize=10000;
  params->simulationType=0;
  params->likelihoodThreshold=0.0;
  params->correlationThreshold=0.0;
  params->ncalls=0;
  strcpy(params->nameOne,"default one");
  strcpy(params->nameTwo,"default two");
  params->start.gpsSeconds=0;
  params->start.gpsNanoSeconds=0;
  params->stop.gpsSeconds=0;
  params->stop.gpsNanoSeconds=0;
  params->oneOutPT=NULL;
  params->twoOutPT=NULL;
  params->oneOutPM=NULL;
  params->twoOutPM=NULL;
  params->oneOutPS=NULL;
  params->twoOutPS=NULL;
  params->coincidenceOut=NULL;
  params->oneOutCluster=NULL;
  params->twoOutCluster=NULL;
  params->oneIn->type=params->twoIn->type=BIORTHOGONAL;
  params->oneIn->treeType=params->twoIn->treeType=BINARY;
  params->oneIn->level=params->twoIn->level=5;
  params->oneIn->border=params->twoIn->border=B_POLYNOM;
  params->oneIn->HPFilterLength=params->twoIn->HPFilterLength=10;
  params->oneIn->LPFilterLength=params->twoIn->LPFilterLength=10;
  params->o1=FALSE;
  params->o2=FALSE;
  params->p1=FALSE;
  params->p2=FALSE;
  params->co1=FALSE;
  params->co2=FALSE;
  params->cl1=TRUE;
  params->cl2=TRUE;
  params->blobP=FALSE;
  params->blobO=FALSE;
  strcpy(params->outputToFile,"PT");
}



void LALInitSearch(LALStatus *status,
		   void **searchParams,
		   LALInitSearchParams *initSearchParams
		   )
{ 
  INT4 ac;
  CHAR **av;
  INT4 i;
  WaveParams *params;
  int pair;

  INITSTATUS( status, "LALInitSearch", INITC );
  ATTATCHSTATUSPTR( status );

  /*Memory allocation*/

  params=(WaveParams *)LALMalloc(sizeof(WaveParams));


  LALAllocateWavelet(status->statusPtr, &params->oneIn);
  CHECKSTATUSPTR(status);
  LALAllocateWavelet(status->statusPtr, &params->twoIn);
  CHECKSTATUSPTR(status);

  _defaults(params);

  params->oneOutPT=
    (OutputPercentileWavelet*)LALCalloc(1,sizeof(OutputPercentileWavelet));
  params->twoOutPT=
    (OutputPercentileWavelet*)LALCalloc(1,sizeof(OutputPercentileWavelet));
  params->oneOutPS=(OutputPixelSwapWavelet*)LALCalloc(1,sizeof(OutputPixelSwapWavelet));
  params->twoOutPS=(OutputPixelSwapWavelet*)LALCalloc(1,sizeof(OutputPixelSwapWavelet));
  params->oneOutPM=(OutputPixelMixerWavelet*)LALCalloc(1,sizeof(OutputPixelMixerWavelet));
  params->twoOutPM=(OutputPixelMixerWavelet*)LALCalloc(1,sizeof(OutputPixelMixerWavelet));
  params->coincidenceOut=(OutputCoincidenceWavelet *)LALCalloc(1,sizeof(OutputCoincidenceWavelet));
  params->coincidenceOutPS=(OutputCoincidenceWavelet *)LALCalloc(1,sizeof(OutputCoincidenceWavelet));
  params->coincidenceOutPM=(OutputCoincidenceWavelet *)LALCalloc(1,sizeof(OutputCoincidenceWavelet));
  params->oneOutCluster=(OutputClusterWavelet *)LALCalloc(1,sizeof(OutputClusterWavelet));
  params->oneOutClusterPS=(OutputClusterWavelet *)LALCalloc(1,sizeof(OutputClusterWavelet));
  params->oneOutClusterPM=(OutputClusterWavelet *)LALCalloc(1,sizeof(OutputClusterWavelet));
  params->twoOutCluster=(OutputClusterWavelet *)LALCalloc(1,sizeof(OutputClusterWavelet));
  params->twoOutClusterPS=(OutputClusterWavelet *)LALCalloc(1,sizeof(OutputClusterWavelet));
  params->twoOutClusterPM=(OutputClusterWavelet *)LALCalloc(1,sizeof(OutputClusterWavelet));

  *searchParams=params;

  /*Default parameters are overwritten if specified in -filterparams */

  ac=initSearchParams->argc;
  av=initSearchParams->argv;

  for(i=1;i<ac;i++)
    {
      pair=0;
      if(strcmp(av[i],"-WaveletType")==0)
	{
	  params->oneIn->type=params->twoIn->type=
	    (INT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-WaveletTreeType")==0)
	{
	  params->oneIn->treeType=params->twoIn->treeType=
	    (INT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-WaveletLevel")==0)
	{
	  params->oneIn->level=
	    params->twoIn->level=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-WaveletBorder")==0)
	{
	  params->oneIn->border=params->twoIn->border=
	    (INT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-WaveletHPFilterLength")==0)
	{
	  params->oneIn->HPFilterLength=
	    params->twoIn->HPFilterLength=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-WaveletLPFilterLength")==0)
	{
	  params->oneIn->LPFilterLength=
	    params->twoIn->LPFilterLength=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-ch1")==0)
	{
	  strcpy(params->nameOne,av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-ch2")==0)
	{
	  strcpy(params->nameTwo,av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-nonZeroFraction")==0)
	{
	  params->nonZeroFraction=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-seed")==0)
	{
	  params->seed=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-timeWindowNanoSec")==0)
	{
	  params->timeWindowNanoSec=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-simulationType")==0)
	{
	  params->simulationType=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-halo")==0)
	{
	  params->aura=(BOOLEAN)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-likelihoodThreshold")==0)
	{
	  params->likelihoodThreshold=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-correlationThreshold")==0)
	{
	  params->correlationThreshold=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-minClusterSize")==0)
	{
	  params->minClusterSize=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-maxClusterSize")==0)
	{
	  params->maxClusterSize=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-pixelMixer")==0)
	{
	  params->usePixelMixer=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE;
	  pair++;
	}
      if(strcmp(av[i],"-pixelSwapOne")==0)
	{
	  params->usePixelSwapOne=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE;
	  pair++;
	}
      if(strcmp(av[i],"-pixelSwapTwo")==0)
	{
	  params->usePixelSwapTwo=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE;
	  pair++;
	}
      if(strcmp(av[i],"-o1")==0)
	{
	  params->o1=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-o2")==0)
	{
	  params->o2=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-p1")==0)
	{
	  params->p1=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-p2")==0)
	{
	  params->p2=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-co1")==0)
	{
	  params->co1=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-co2")==0)
	{
	  params->co2=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-cl1")==0)
	{
	  params->cl1=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-cl2")==0)
	{
	  params->cl2=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-blobP")==0)
	{
	  params->blobP=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE;
	  pair++;
	}
      if(strcmp(av[i],"-blobO")==0)
	{
	  params->blobO=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE;
	  pair++;
	}
      if(strcmp(av[i],"-outputToFile")==0)
	{
	  strcpy(params->outputToFile,av[i+1]);
	  pair++;
	}
    }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

